@@ -1,10 +1,10 @@
 package XML::RSS;
+
 use strict;
+use warnings;
+
 use Carp;
 use XML::Parser;
-use HTML::Entities qw(encode_entities_numeric encode_entities);
-use DateTime::Format::Mail;
-use DateTime::Format::W3CDTF;
 
 use XML::RSS::Private::Output::Base;
 use XML::RSS::Private::Output::V0_9;
@@ -14,7 +14,9 @@ use XML::RSS::Private::Output::V2_0;
 
 use vars qw($VERSION $AUTOLOAD @ISA $AUTO_ADD);
 
-$VERSION = '1.33';
+require 5.008;
+
+$VERSION = '1.47';
 
 $AUTO_ADD = 0;
 
@@ -22,9 +24,9 @@ sub _get_ok_fields {
     return {
         "0.9" => {
             channel => {
-                title       => '',
-                description => '',
-                link        => '',
+                title       => undef,
+                description => undef,
+                link        => undef,
             },
             image => {
                 title => undef,
@@ -40,13 +42,13 @@ sub _get_ok_fields {
         },
         "0.91" => {
             channel => {
-                title          => '',
+                title          => undef,
                 copyright      => undef,
-                description    => '',
+                description    => undef,
                 docs           => undef,
                 language       => undef,
                 lastBuildDate  => undef,
-                'link'         => '',
+                'link'         => undef,
                 managingEditor => undef,
                 pubDate        => undef,
                 rating         => undef,
@@ -71,9 +73,9 @@ sub _get_ok_fields {
         },
         "2.0" => {
             channel => {
-                title          => '',
-                'link'         => '',
-                description    => '',
+                title          => undef,
+                'link'         => undef,
+                description    => undef,
                 language       => undef,
                 copyright      => undef,
                 managingEditor => undef,
@@ -109,9 +111,9 @@ sub _get_ok_fields {
         },
         'default' => {
             channel => {
-                title       => '',
-                description => '',
-                link        => '',
+                title       => undef,
+                description => undef,
+                link        => undef,
             },
             image => {
                 title => undef,
@@ -247,7 +249,8 @@ sub _rdf_resource_fields {
     };
 }
 
-my %empty_ok_elements = (enclosure => 1,);
+my %empty_ok_elements = (enclosure => 1);
+my %hashref_ok_elements = (description => 1);
 
 sub _get_default_modules {
     return {
@@ -306,6 +309,7 @@ sub _get_init_default_key_assignments {
         {key => "output",        default => "",},
         {key => "encoding",      default => "UTF-8",},
         {key => "encode_cb",     default => undef(),},
+        {key => "xml:base",      default => undef(),},
     ];
 }
 
@@ -324,6 +328,8 @@ sub _reset {
     # initialize items
     $self->{items} = [];
 
+    delete $self->{_allow_multiple};
+
     my $ok_fields = $self->_get_ok_fields();
 
     my $ver_ok_fields =
@@ -348,7 +354,6 @@ sub _initialize {
     # namespaces
     $self->{namespaces}    = {};
     $self->{rss_namespace} = '';
-
     foreach my $k (@{$self->_get_init_default_key_assignments()})
     {
         my $key = $k->{key};
@@ -380,8 +385,8 @@ sub add_module {
     my $self = shift;
     my $hash = {@_};
 
-    $hash->{prefix} =~ /^[a-z_][a-z0-9.\-_]*$/
-      or croak "a namespace prefix should look like [a-z_][a-z0-9.\\-_]*";
+    $hash->{prefix} =~ /^[a-z_][a-z0-9.\-_]*$/i
+      or croak "a namespace prefix should look like [A-Za-z_][A-Za-z0-9.\\-_]*";
 
     $hash->{uri}
       or croak "a URI must be provided in a namespace declaration";
@@ -530,6 +535,28 @@ sub _get_output_version {
     return ($self->{output} =~ /\d/) ? $self->{output} : $self->{version};
 }
 
+# This is done to preserve backwards compatibility with older versions
+# of XML-RSS that had the channel/{link,description,title} as the empty
+# string by default.
+sub _output_env {
+    my $self = shift;
+    my $callback = shift;
+
+    local $self->{channel}->{'link'} = $self->{channel}->{'link'};
+    local $self->{channel}->{'description'} = $self->{channel}->{'description'};
+    local $self->{channel}->{'title'} = $self->{channel}->{'title'};
+
+    foreach my $field (qw(link description title))
+    {
+        if (!defined($self->{channel}->{$field}))
+        {
+            $self->{channel}->{$field} = '';
+        }
+    }
+
+    return $callback->();
+}
+
 sub as_string {
     my $self = shift;
 
@@ -537,7 +564,9 @@ sub as_string {
 
     my $output_method = $self->_get_output_method($version);
 
-    return $self->$output_method();
+    return $self->_output_env(
+        sub { return $self->$output_method(); }
+    );
 }
 
 # Checks if inside a possibly namespaced element
@@ -603,8 +632,23 @@ sub _is_rdf_resource {
     );
 }
 
+sub _get_ns_arrayity {
+    my ($self, $ns) = @_;
+
+    my $is_array =
+           $self->_parse_options()->{'modules_as_arrays'}
+        && (!exists($self->_get_default_modules()->{$ns}))
+        # RDF
+        && ($ns ne "http://www.w3.org/1999/02/22-rdf-syntax-ns#")
+        ;
+
+    my $default_ref = sub { $is_array ? [] : {} };
+
+    return ($is_array, $default_ref);
+}
+
 sub _append_text_to_elem_struct {
-    my ($self, $struct, $cdata, $mapping_sub) = @_;
+    my ($self, $struct, $cdata, $mapping_sub, $is_array_sub) = @_;
 
     my $elem = $self->_current_element;
 
@@ -612,26 +656,73 @@ sub _append_text_to_elem_struct {
 
     # If it's in the default namespace
     if ($verdict) {
-        $struct->{$mapping_sub->($struct, $elem)} .= $cdata;
+        $self->_append_struct(
+            $struct,
+            scalar($mapping_sub->($struct, $elem)),
+            scalar($is_array_sub->($struct, $elem)),
+            $cdata
+        );
     }
     else {
-        # If it's in another namespace
-        $struct->{$ns}->{$elem} .= $cdata;
+        my $prefix = $self->{modules}->{$ns};
+
+        my ($is_array, $default_ref) = $self->_get_ns_arrayity($ns);
+
+        $self->_append_struct(
+            ($struct->{$ns} ||= $default_ref->()),
+            $elem,
+            (defined($prefix) && $prefix eq "dc"),
+            $cdata
+        );
 
         # If it's in a module namespace, provide a friendlier prefix duplicate
-        if ($self->{modules}->{$ns}) {
-            $struct->{$self->{modules}->{$ns}}->{$elem} .= $cdata;
+        if ($prefix) {
+            $self->_append_struct(
+                ($struct->{$prefix} ||= $default_ref->()),
+                $elem,
+                ($prefix eq "dc"),
+                $cdata
+            );
         }
     }
 
     return;
 }
 
+sub _append_struct {
+    my ($self, $struct, $key, $can_be_array, $cdata) = @_;
+
+    if (ref($struct) eq 'ARRAY') {
+        $struct->[-1]->{'val'} .= $cdata;
+        return;
+    }
+    elsif (defined $struct->{$key}) {
+        if (ref($struct->{$key}) eq 'HASH') {
+            $struct->{$key}->{content} .= $cdata;
+            return;
+        }
+        elsif ($can_be_array && ref($struct->{$key}) eq 'ARRAY') {
+            $struct->{$key}->[-1] .= $cdata;
+            return;
+        }
+    }
+
+    $struct->{$key} .= $cdata;
+    return;
+}
+
 sub _return_elem {
     my ($struct, $elem) = @_;
     return $elem;
 }
 
+sub _return_elem_is_array {
+    my ($struct, $elem) = @_;
+
+    # Always return false because no element should be an array.
+    return;
+}
+
 sub _append_text_to_elem {
     my ($self, $ext_tag, $cdata) = @_;
 
@@ -639,6 +730,7 @@ sub _append_text_to_elem {
         $self->$ext_tag(),
         $cdata,
         \&_return_elem,
+        \&_return_elem_is_array,
     );
 }
 
@@ -664,6 +756,12 @@ sub _return_item_elem {
     }
 }
 
+sub _return_item_elem_is_array {
+    my ($item, $elem) = @_;
+
+    return ($elem eq "category");
+}
+
 sub _append_text_to_item {
     my ($self, $cdata) = @_;
 
@@ -672,11 +770,33 @@ sub _append_text_to_item {
     }
 
     $self->_append_text_to_elem_struct(
-        $self->{'items'}->[$self->{num_items} - 1],
+        $self->_last_item,
         $cdata,
         \&_return_item_elem,
+        \&_return_item_elem_is_array
     );
 }
+
+sub _append_to_array_elem {
+    my ($self, $category, $cdata) = @_;
+
+    if (! $self->_my_in_element($category))
+    {
+        return;
+    }
+
+    my $el = $self->_current_element;
+
+    if (ref($self->{$category}->{$el}) eq "ARRAY") {
+        $self->{$category}->{$el}->[-1] .= $cdata;
+    }
+    else {
+        $self->{$category}->{$el} .= $cdata;
+    }
+
+    return 1;
+}
+
 sub _handle_char {
     my ($self, $cdata) = (@_);
 
@@ -698,20 +818,24 @@ sub _handle_char {
         $self->_append_text_to_elem("textinput", $cdata);
     }
     # skipHours element
-    elsif ($self->_my_in_element("skipHours")) {
-        $self->{'skipHours'}->{$self->_current_element} .= $cdata;
-
+    elsif ($self->_append_to_array_elem("skipHours", $cdata)) {
+        # Do nothing - already done in the predicate.
     }
-    # skipDays element
-    elsif ($self->_my_in_element("skipDays")) {
-        $self->{'skipDays'}->{$self->_current_element} .= $cdata;
-
+    elsif ($self->_append_to_array_elem("skipDays", $cdata)) {
+        # Do nothing - already done in the predicate.
     }
     # channel element
     elsif ($self->_my_in_element("channel")) {
-        return if $self->_within_topics;
+        if ($self->_within_topics() || $self->_my_in_element("items")) {
+            return;
+        }
 
-        $self->_append_text_to_elem("channel", $cdata);
+        if ($self->_current_element eq "category") {
+            $self->_append_to_array_elem("channel", $cdata);
+        }
+        else {
+            $self->_append_text_to_elem("channel", $cdata);
+        }
     }
 }
 
@@ -722,7 +846,73 @@ sub _handle_dec {
     #print "ENCODING: $encoding\n";
 }
 
+sub _should_be_hashref {
+    my ($self, $el) = @_;
 
+    return
+    (
+        $empty_ok_elements{$el}
+        || ($self->_parse_options()->{'hashrefs_instead_of_strings'}
+            && $hashref_ok_elements{$el}
+        )
+    );
+}
+
+sub _start_array_element_in_struct {
+    my ($self, $input_struct, $el, $prefix) = @_;
+
+    my ($el_ns, $el_verdict) = $self->_get_elem_namespace($el);
+
+    my ($is_array, $default_ref) = $self->_get_ns_arrayity($el_ns);
+
+    my @structs = (!$el_verdict)
+        ? (
+            (exists($self->{modules}->{$el_ns})
+                ? ($input_struct->{$self->{modules}->{$el_ns}} ||= $default_ref->())
+                : ()
+            ),
+            ($input_struct->{$el_ns} ||= $default_ref->()),
+        )
+        : ($input_struct)
+        ;
+
+    foreach my $struct (@structs)
+    {
+        if (ref($struct) eq 'ARRAY') {
+            push @$struct, { el => $el, val => "", };
+        }
+        # If it's an array - append a new empty element because a new one
+        # was started.
+        elsif (ref($struct->{$el}) eq "ARRAY") {
+            push @{$struct->{$el}}, "";
+        }
+        # If it's not an array but still full (i.e: it's only the second
+        # element), then turn it into an array
+        elsif (defined($struct->{$el}) && length($struct->{$el})) {
+            $struct->{$el} = [$struct->{$el}, ""];
+        }
+        # Else - do nothing and let the function append to the new value
+        #
+    }
+    return 1;
+}
+
+sub _start_array_element {
+    my ($self, $cat, $el) = @_;
+
+    if (!$self->_my_in_element($cat)) {
+        return;
+    }
+
+    $self->_start_array_element_in_struct($self->{$cat}, $el);
+    return 1;
+}
+
+sub _last_item {
+    my $self = shift;
+
+    return ($self->{'items'}->[$self->{num_items} - 1] ||= {});
+}
 
 sub _handle_start {
     my $self    = shift;
@@ -731,6 +921,16 @@ sub _handle_start {
 
     my $parser = $self->_parser;
 
+    my ($el_ns, $el_verdict) = $self->_get_elem_namespace($el);
+    
+    if ($el eq "image")
+    {
+        if (exists($attribs{'resource'}))
+        {
+            $self->image("rdf:resource", $attribs{'resource'});
+        }
+    }
+
     # beginning of RSS 0.91
     if ($el eq 'rss') {
         if (exists($attribs{version})) {
@@ -740,7 +940,10 @@ sub _handle_start {
             croak "Malformed RSS: invalid version\n";
         }
 
-        # beginning of RSS 1.0 or RSS 0.9
+        # handle xml:base
+        $self->{'xml:base'} = $attribs{'base'} if exists $attribs{'base'};
+
+    # beginning of RSS 1.0 or RSS 0.9
     }
     elsif ($el eq 'RDF') {
         my @prefixes = $parser->new_ns_prefixes;
@@ -779,7 +982,16 @@ sub _handle_start {
         #    croak "Malformed RSS: invalid version\n";
         #}
 
-        # beginning of item element
+        # handle xml:base
+        $self->{'xml:base'} = $attribs{'base'} if exists $attribs{'base'};
+
+    # beginning of item element
+    }
+    elsif ($self->_start_array_element("skipHours", $el)) {
+        # Do nothing - already done in the predicate.
+    }
+    elsif ($self->_start_array_element("skipDays", $el)) {
+        # Do nothing - already done in the predicate.
     }
     elsif ($el eq 'item') {
 
@@ -798,11 +1010,14 @@ sub _handle_start {
                 $self->{_inside_item_elem} = $parser->depth();
             }
         }
+        # handle xml:base
+        $self->_last_item->{'xml:base'} = $attribs{'base'} if exists $attribs{'base'};
+
 
         # guid element is a permanent link unless isPermaLink attribute is set to false
     }
     elsif ($el eq 'guid') {
-        $self->{'items'}->[$self->{num_items} - 1]->{'isPermaLink'} =
+        $self->_last_item->{'isPermaLink'} =
           (exists($attribs{'isPermaLink'}) && 
               (lc($attribs{'isPermaLink'}) eq 'true')
           );
@@ -810,6 +1025,17 @@ sub _handle_start {
         # beginning of taxo li element in item element
         #'http://purl.org/rss/1.0/modules/taxonomy/' => 'taxo'
     }
+    elsif (
+           $self->_current_element eq "item"
+        && (($el eq "category") || 
+            (
+                   exists($self->{modules}->{$el_ns})
+                && ($self->{modules}->{$el_ns} eq "dc")
+            )
+        )
+    ) {
+        $self->_start_array_element_in_struct($self->_last_item, $el);
+    }
     elsif (
         $parser->within_element(
             $parser->generate_ns_name("topics", 'http://purl.org/rss/1.0/modules/taxonomy/')
@@ -821,7 +1047,7 @@ sub _handle_start {
     {
 
         #print "taxo: ", $attribs{'resource'},"\n";
-        push(@{$self->{'items'}->[$self->{num_items} - 1]->{'taxo'}}, $attribs{'resource'});
+        push(@{$self->_last_item->{'taxo'}}, $attribs{'resource'});
         $self->{'modules'}->{'http://purl.org/rss/1.0/modules/taxonomy/'} = 'taxo';
 
         # beginning of taxo li in channel element
@@ -865,7 +1091,6 @@ sub _handle_start {
             }
         }
     }
-
     # beginning of an item element that stores its info in rdf:resource
     elsif ( $parser->namespace($el)
         && $self->_is_rdf_resource($el)
@@ -877,23 +1102,93 @@ sub _handle_start {
         # in the 'rdf_resource_fields' so this condition always evaluates
         # to false.
         # if ( $ns eq $self->{rss_namespace} ) {
-        #   $self->{'items'}->[$self->{num_items}-1]->{ $el } = $attribs{resource};
+        #   $self->_last_item->{ $el } = $attribs{resource};
         # }
         # else
         {
-            $self->{'items'}->[$self->{num_items} - 1]->{$ns}->{$el} = $attribs{resource};
+            $self->_last_item->{$ns}->{$el} = $attribs{resource};
 
             # add short cut
             #
             if (exists($self->{modules}->{$ns})) {
                 $ns = $self->{modules}->{$ns};
-                $self->{'items'}->[$self->{num_items} - 1]->{$ns}->{$el} = $attribs{resource};
+                $self->_last_item->{$ns}->{$el} = $attribs{resource};
             }
         }
     }
-    elsif ($empty_ok_elements{$el} and $self->_current_element eq 'item') {
-        $self->{items}->[$self->{num_items} - 1]->{$el} = \%attribs;
+    elsif ($self->_should_be_hashref($el) and $self->_current_element eq 'item') {
+        if (defined $attribs{base}) {
+            $attribs{'xml:base'} = delete $attribs{base};
+        }
+        if (keys(%attribs)) {
+            if ($el_verdict) {
+                $self->_last_item->{$el} =
+                  $self->_make_array($el, $self->_last_item->{$el}, \%attribs);
+            }
+            else {
+                $self->_last_item->{$el_ns}->{$el} =
+                  $self->_make_array($el, $self->_last_item->{$el_ns}->{$el}, \%attribs);
+
+                my $prefix = $self->{modules}->{$el_ns};
+
+                if ($prefix) {
+                    $self->_last_item->{$prefix}->{$el} =
+                      $self->_make_array($el, $self->_last_item->{$prefix}->{$el}, \%attribs);
+                }
+            }
+        }
     }
+    elsif ($self->_start_array_element("image", $el)) {
+        # Do nothing - already done in the predicate.
+    }    
+    elsif (($el eq "category") &&
+        (!$parser->within_element("item")) &&
+        $self->_start_array_element("channel", $el)) {
+        # Do nothing - already done in the predicate.
+    }
+    elsif (($self->_current_element eq 'channel') &&
+           ($el_verdict))
+           {
+        # Make sure an opening tag signifies that the element has been
+        # encountered.
+        if (   exists($self->{'channel'}->{$el}) 
+            && (!defined($self->{'channel'}->{$el})))
+        {
+            $self->{'channel'}->{$el} = "";
+        }
+    }
+}
+
+sub _make_array {
+    my $self = shift;
+    my $el   = shift;
+    my $old  = shift;
+    my $new  = shift;
+
+    if (!$self->_allow_multiple($el)) {
+      return $new;
+    }
+
+    if (!defined $old) {
+        $old = [];
+    } elsif (ref($old) ne 'ARRAY') {
+        $old = [$old];
+    }
+    push @$old, $new;
+    return $old;
+}
+
+sub _allow_multiple {
+    my $self = shift;
+    my $el   = shift;
+
+    $self->{_allow_multiple} ||=
+        {
+            map { $_ => 1 }
+            @{$self->_parse_options->{allow_multiple} || []}
+        };
+
+    return $self->{_allow_multiple}->{$el};
 }
 
 sub _handle_end {
@@ -971,56 +1266,93 @@ sub _get_parser {
     );    
 }
 
+sub _parse_options {
+    my $self = shift;
+
+    if (@_) {
+        $self->{_parse_options} = shift;
+    }
+
+    return $self->{_parse_options};
+}
+
+sub _empty {}
+
+sub _generic_parse {
+    my $self = shift;
+    my $method = shift;
+    my $arg = shift;
+    my $options = shift;
+
+    $self->_reset;
+
+    $self->_parse_options($options || {});
+
+    # Workaround to make sure that if we were defined with version => "2.0"
+    # then we can still parse 1.0 and 0.9.x feeds correctly.
+    if ($self->{version} eq "2.0") {
+        $self->{modules} = +{%{$self->_get_default_modules()}, %{$self->{modules}}};
+    }
+
+    {
+        my $parser = $self->_get_parser();
+
+        eval {
+            $parser->$method($arg);
+        };
+
+        if ($@)
+        {
+            my $err = $@;
+
+            # Cleanup so perl-5.6.2 will be happy.
+            $parser->setHandlers(
+                map { ($_ => \&_empty) } (qw(Char XMLDecl Start End))
+            );
+            $self->_parser(0);
+
+            undef($parser);
+
+            die $err;
+        }
+    }
+
+    $self->_auto_add_modules if $AUTO_ADD;
+    $self->{version} = $self->{_internal}->{version};
+
+    return $self;
+}
 
 sub parse {
     my $self = shift;
     my $text_to_parse = shift;
+    my $options = shift;
 
-    $self->_reset;
-
-    # Workaround to make sure that if we were defined with version => "2.0"
-    # then we can still parse 1.0 and 0.9.x feeds correctly.
-    if ($self->{version} eq "2.0") {
-        $self->{modules} = +{%{$self->_get_default_modules()}, %{$self->{modules}}};
-    }
-
-    $self->_get_parser()->parse($text_to_parse);
-
-    $self->_auto_add_modules if $AUTO_ADD;
-    $self->{version} = $self->{_internal}->{version};
+    return $self->_generic_parse("parse", $text_to_parse, $options);
 }
 
 sub parsefile {
     my $self = shift;
+    my $file_to_parse = shift;
+    my $options = shift;
 
-    $self->_reset;
-
-    # Workaround to make sure that if we were defined with version => "2.0"
-    # then we can still parse 1.0 and 0.9.x feeds correctly.
-    if ($self->{version} eq "2.0") {
-        $self->{modules} = +{%{$self->_get_default_modules()}, %{$self->{modules}}};
-    }
-
-    $self->_get_parser()->parsefile(shift);
-
-    $self->_auto_add_modules if $AUTO_ADD;
-    $self->{version} = $self->{_internal}->{version};
+    return $self->_generic_parse("parsefile", $file_to_parse, $options);
 }
 
-# Check if Perl supports the :encoding layer in File I/O.
-sub _perl_supports_encoding {
+sub _untaint {
     my $self = shift;
 
-    return $] > 5.007;
+    my $value = shift;
+    
+    my ($untainted) = ($value =~ m{(.*)}s);
+
+    return $untainted;
 }
 
 sub _get_save_output_mode {
     my $self = shift;
 
-    return $self->_perl_supports_encoding()
-        ? (">:encoding(" . $self->_encoding() . ")")
-        : ">"
-        ;
+    return (">:encoding(" . $self->_untaint($self->_encoding()) . ")");
 }
 
 sub save {
@@ -1166,7 +1498,7 @@ XML::RSS - creates and updates RSS files
 
  # create an RSS 1.0 file (http://purl.org/rss/1.0/)
  use XML::RSS;
- my $rss = new XML::RSS (version => '1.0');
+ my $rss = XML::RSS->new(version => '1.0');
  $rss->channel(
    title        => "freshmeat.net",
    link         => "http://freshmeat.net",
@@ -1238,7 +1570,7 @@ XML::RSS - creates and updates RSS files
 
  # create an RSS 2.0 file
  use XML::RSS;
- my $rss = new XML::RSS (version => '2.0');
+ my $rss = XML::RSS->new (version => '2.0');
  $rss->channel(title          => 'freshmeat.net',
                link           => 'http://freshmeat.net',
                language       => 'en',
@@ -1277,7 +1609,7 @@ XML::RSS - creates and updates RSS files
 
  # create an RSS 0.9 file
  use XML::RSS;
- my $rss = new XML::RSS (version => '0.9');
+ my $rss = XML::RSS->new( version => '0.9' );
  $rss->channel(title => "freshmeat.net",
                link  => "http://freshmeat.net",
                description => "the one-stop-shop for all your Linux software needs",
@@ -1304,11 +1636,16 @@ XML::RSS - creates and updates RSS files
  # or save it to a file
  $rss->save("fm.rdf");
 
- # insert an item into an RSS file and removes the oldest item if
- # there are already 15 items
- my $rss = new XML::RSS;
+ # insert an item into an RSS file and removes the oldest ones if
+ # there are already 15 items or more
+ my $rss = XML::RSS->new;
  $rss->parsefile("fm.rdf");
- shift(@{$rss->{'items'}}) if (@{$rss->{'items'}} == 15);
+
+ while (@{$rss->{'items'}} >= 15)
+ {
+     pop(@{$rss->{'items'});
+ }
+
  $rss->add_item(title => "MpegTV Player (mtv) 1.0.9.7",
                 link  => "http://freshmeat.net/news/1999/06/21/930003958.html",
                 mode  => 'insert'
@@ -1348,17 +1685,21 @@ adopted it as a simple syndication format. With the advent of RSS 1.0,
 users are now able to syndication many different kinds of content
 including news headlines, threaded measages, products catalogs, etc.
 
+B<Note:> In order to parse and generate dates (such as C<pubDate>
+and C<dc:date>) it is recommended to use L<DateTime::Format::Mail> and 
+L<DateTime::Format::W3CDTF> , which is what L<XML::RSS> uses internally
+and requires.
+
 =head1 METHODS
 
 =over 4
 
-=item new XML::RSS (version=>$version, encoding=>$encoding,
-output=>$output, stylesheet=>$stylesheet_url)
+=item XML::RSS->new(version=>$version, encoding=>$encoding, output=>$output, stylesheet=>$stylesheet_url, 'xml:base'=>$base)
 
 Constructor for XML::RSS. It returns a reference to an XML::RSS object.
 You may also pass the RSS version and the XML encoding to use. The default
 B<version> is 1.0. The default B<encoding> is UTF-8. You may also specify
-the B<output> format regarless of the input version. This comes in handy
+the B<output> format regardless of the input version. This comes in handy
 when you want to convert RSS between versions. The XML::RSS modules
 will convert between any of the formats.  If you set <encode_output> XML::RSS
 will make sure to encode any entities in generated RSS.  This is now on by
@@ -1375,6 +1716,10 @@ should normally not concern you) and the text to encode. It should return
 the text to encode. If not set, then the module will encode using its
 custom encoding routine.
 
+xml:base will set an C<xml:base> property as per
+
+    http://www.w3.org/TR/xmlbase/
+
 Note that in order to encode properly, you need to handle "CDATA" sections
 properly. Look at L<XML::RSS::Private::Output::Base>'s C<_default_encode()>
 method for how to do it properly.
@@ -1419,14 +1764,46 @@ in RSS 0.91 or optionally imported into RSS 1.0 via the rss091 namespace.
 The method for retrieving the values for the image is the same as it
 is for B<channel()>.
 
-=item parse ($string)
+=item parse ($string, \%options)
 
-Parses an RDF Site Summary which is passed into B<parse()> as the first parameter.
+Parses an RDF Site Summary which is passed into B<parse()> as the first 
+parameter. Returns the instance of the object so one can say 
+C<<$rss->parse($string)->other_method()>>.
 
 See the add_module() method for instructions on automatically adding
 modules as a string is parsed.
 
-=item parsefile ($file)
+%options is a list of options that specify how parsing is to be done. The
+available options are:
+
+=over 4
+
+=item * allow_multiple
+
+Takes an array ref of names which indicates which elements should
+be allowed to have multiple occurrences. So, for example, to parse
+feeds with multiple enclosures
+
+   $rss->parse($xml, { allow_multiple => ['enclosure'] });
+
+=item * hashrefs_instead_of_strings
+
+If true, then some items (so far "C<description>") will become hash-references
+instead of strings (with a B<content> key containing their content , B<if>
+they have XML attributes. Without this key, the attributes will be ignored
+and there will only be a string. Thus, specifying this option may break
+compatibility.
+
+=item * modules_as_arrays
+
+This option when true, will parse the modules key-value-pairs as an arrayref of 
+C<<< { el => $key_name, value => $value, } >>> hash-refs to gracefully
+handle duplicate items (see below). It will not affect the known modules such 
+as dc ("Dublin Core").
+
+=back
+
+=item parsefile ($file, \%options)
 
 Same as B<parse()> except it parses a file rather than a string.
 
@@ -1495,7 +1872,7 @@ attributes such as title, link, and description.  The only difference
 is the compartmentalization of their key/value paris in a second-level
 hash.
 
-  $rss->add_item (title=>$title, link=>$link, dc=>{ subject=>$subject, creator=>$creator });
+  $rss->add_item (title=>$title, link=>$link, dc=>{ subject=>$subject, creator=>$creator, date=>$date });
 
 For elements of the Dublin Core module, use the key 'dc'.  For elements
 of the Syndication module, 'syn'.  For elements of the Taxonomy module,
@@ -1506,6 +1883,20 @@ They are associated with appropriate URI-based namespaces:
   dc:   http://purl.org/dc/elements/1.1/
   taxo: http://purl.org/rss/1.0/modules/taxonomy/
 
+The Dublin Core ('dc') hash keys may be point to an array
+reference, which in turn will specify multiple such keys, and render them
+one after the other. For example:
+
+    $rss->add_item (
+        title => $title,
+        link => $link,
+        dc => { 
+            subject=> ["Jungle", "Desert", "Swamp"],
+            creator=>$creator,
+            date=>$date
+        },
+    );
+
 Dublin Core elements may occur in channel, image, item(s), and textinput
 -- albeit uncomming to find them under image and textinput.  Syndication
 elements are limited to the channel element. Taxonomy elements can occur
@@ -1535,6 +1926,15 @@ Then proceed as usual:
 
   $rss->add_item (title=>$title, link=>$link, my=>{ rating=>$rating });
 
+You can also set the value of the module's prefix to an array reference 
+of C<<< { el => , val => } >>> hash-references, in which case duplicate 
+elements are possible:
+
+  $rss->add_item(title=>$title, link=>$link, my=> [
+    {el => "rating", value => $rating1, }
+    {el => "rating", value => $rating2, },
+  ]
+
 Non-standard namespaces are not, however, currently accessible via a simple
 prefix; access them via their namespace URL like so:
 
@@ -1629,11 +2029,12 @@ L<http://svn.perl.org/modules/XML-RSS/>
 
 =head1 AUTHOR
 
-    Original code: Jonathan Eisenzopf <eisen@pobox.com>
-    Further changes: Rael Dornfest <rael@oreilly.com>
+Original code: Jonathan Eisenzopf <eisen@pobox.com>
 
-    Currently: Ask Bjoern Hansen <ask@develooper.com> 
+Further changes: Rael Dornfest <rael@oreilly.com>, Ask Bjoern Hansen 
+<ask@develooper.com> 
 
+Currently: Shlomi Fish <shlomif@cpan.org>
 
 =head1 COPYRIGHT
 
@@ -1641,6 +2042,8 @@ Copyright (c) 2001 Jonathan Eisenzopf <eisen@pobox.com> and Rael
 Dornfest <rael@oreilly.com>, Copyright (C) 2006-2007 Ask Bjoern Hansen
 <ask@develooper.com>.
 
+=head1 LICENSE
+
 XML::RSS is free software. You can redistribute it and/or
 modify it under the same terms as Perl itself.
 
@@ -1653,7 +2056,7 @@ modify it under the same terms as Perl itself.
  rjp@browser.org
  Kellan Elliott-McCrea <kellan@protest.net>
  Rafe Colburn <rafe@rafe.us>
- Adam Trickett <adam.trickett@btinternet.com>
+ Adam Trickett <atrickett@cpan.org>
  Aaron Straup Cope <asc@vineyard.net>
  Ian Davis <iand@internetalchemy.org>
  rayg@varchars.com
